<?php

namespace Drupal\Tests\footnotes\FunctionalJavascript;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\FunctionalJavascriptTests\WebDriverTestBase;
use Drupal\Tests\ckeditor5\Traits\CKEditor5TestTrait;
use Drupal\Tests\footnotes\Traits\FootnotesTestHelperTrait;

/**
 * Contains Footnotes CKEditor plugin functionality tests.
 *
 * @group footnotes
 */
class FootnotesCkeditorPluginTest extends WebDriverTestBase {

  use StringTranslationTrait;
  use CKEditor5TestTrait;
  use FootnotesTestHelperTrait;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'node',
    'ckeditor5',
    'filter',
    'ckeditor5_test',
    'footnotes',
    'block',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->setUpFormatAndEditor();
    $this->setUpNodeTypeAndField();
    $this->setUpDefaultUserWithAccess();
  }

  /**
   * Tests CKEditor5 plugin dialog loads.
   */
  public function testUiLoads() {
    $session = $this->getSession();
    $assert_session = $this->assertSession();

    $this->drupalGet("node/add/page");
    $page = $session->getPage();

    $this->waitForEditor();
    $this->assertEditorButtonEnabled('Footnotes');
    $this->pressEditorButton('Footnotes');

    // Open the dialog and check it contains the form
    // fields and nested CK Editor.
    $dialog_div = $this->assertSession()->waitForElementVisible('css', 'div.ui-dialog');
    $this->assertNotNull($dialog_div, 'Footnotes dialog was opened');
    $assert_session->elementTextContains('css', '.ui-dialog', $this->t('Footnote content'));
    $assert_session->elementTextContains('css', '.ui-dialog', $this->t('Footnote value'));
    $this->assertNotEmpty(
      $assert_session->waitForElementVisible('css', '.ui-dialog .ck'),
      'CK Editor 5 loaded within the dialog form'
    );
    $page->find('css', 'button.ui-dialog-titlebar-close')->click();
  }

  /**
   * Tests CKEditor5 plugin dialog loads.
   */
  public function testBasicFootnoteCreation() {
    $assert_session = $this->assertSession();

    $this->drupalGet("node/add/page");

    $this->waitForEditor();
    $this->assertEditorButtonEnabled('Footnotes');
    $this->pressEditorButton('Footnotes');

    // Open the dialog and check it contains the form
    // fields and nested CK Editor.
    $this->assertSession()->waitForElementVisible('css', 'div.ui-dialog');
    $content_area = $assert_session->waitForElementVisible('css', 'div.ui-dialog .ck-editor__editable');
    $content_area->click();
    $content_area->setValue('Lorem ipsum dolor sit amet.');
    $this->assertNotEmpty($content_area->getText());
    $this->click('.ui-dialog-buttonset .form-submit');
    $this->assertSession()->waitForElementRemoved('css', 'div.ui-dialog');
    $this->assertSession()->waitForElementVisible('css', '.ck .footnotes-preview');
    $this->assertSession()->elementTextContains('css', '.ck .footnotes-preview', 'Lorem ipsum dolor sit amet.');
    $this->assertSession()->elementTextContains('css', '.ck .footnotes-preview', '[#]');

    // Ensure that the main ck editor area now contains the footnote preview.
    $content_area = $assert_session->waitForElementVisible('css', '.field--name-body .ck-editor__editable');
    $this->assertNotEmpty($content_area->getText());
    $this->htmlOutput($content_area->getText());
    $this->assertSession()->elementTextContains('css', '.footnote-content-ckeditor-preview', 'Lorem ipsum dolor sit amet.');
    $this->assertSession()->elementTextContains('css', '.footnotes-preview', '[#]');

    // Test double-click to change the text.
    $this->getSession()->getDriver()->doubleClick("//*[contains(@class, 'ck')]//span[contains(@class, 'footnotes-preview')]");
    $this->assertSession()->waitForElementVisible('css', 'div.ui-dialog');
    $content_area = $assert_session->waitForElementVisible('css', 'div.ui-dialog .ck-editor__editable');
    $content_area->click();
    $content_area->setValue('Added text.');
    $this->assertNotEmpty($content_area->getText());
    $this->click('.ui-dialog-buttonset .form-submit');
    $this->assertSession()->waitForElementRemoved('css', 'div.ui-dialog');
    $this->assertSession()->waitForElementVisible('css', '.ck .footnotes-preview');
    $this->assertSession()->elementTextContains('css', '.ck .footnotes-preview', 'Lorem ipsum dolor sit amet.Added text');
    $this->assertSession()->elementTextContains('css', '.ck .footnotes-preview', '[#]');
  }

}
